package speiger.src.coreengine.rendering.input.divices;

import java.util.function.BiPredicate;

import org.lwjgl.glfw.GLFW;

import speiger.src.coreengine.rendering.input.divices.Keyboard.KeyTask;
import speiger.src.coreengine.rendering.input.divices.Keyboard.KeyboardData;
import speiger.src.coreengine.rendering.input.window.Window;


public class Keyboard extends AbstractDevice<KeyboardData, KeyTask> {
	@Override
	public void reset(long windowId) {}
	@Override
	protected KeyboardData createData(long windowId) { return new KeyboardData(); }
	@Override
	protected void process(KeyTask task) {}
	
	@Override
	protected void registerCallbacks(Window window) {
		super.registerCallbacks(window);
		window.addCallback(this::keyPress, GLFW::glfwSetKeyCallback);
		window.addCallback(this::charTyped, GLFW::glfwSetCharCallback);
	}
	
	private void keyPress(long window, int key, int scancode, int action, int mods) { push(window, new KeyPressed(window, key, scancode, action, mods)); }
	
	private void charTyped(long window, int codepoint) { push(window, new CharTyped(window, codepoint)); }
	
	public static class KeyboardData {
		BiPredicate<Integer, Keyboard> filter;
		boolean[] pressedKeys = new boolean[350];
		boolean[] nonConsumedKeys = new boolean[350];
		
	}
	
	public static interface KeyTask {
		public void process(Keyboard board);
	}
	
	public record KeyPressed(long window, int key, int scancode, int action, int mods) implements KeyTask {
		@Override
		public void process(Keyboard board) {}
	}
	
	public record CharTyped(long window, int codepoint) implements KeyTask {
		@Override
		public void process(Keyboard board) {}
	}
}
